# -*- coding: utf-8 -*-
"""
pyplyn.elements
~~~~~~~~~~

This module implements the Pyplyn Fundamental Elements

Author: Taha Dogan Gunes
License: MIT, see LICENSE for more details.

"""
import abc


class PypElement(object):
    __metaclass__ = abc.ABCMeta


class MidPypElement(PypElement):
    __metaclass__ = abc.ABCMeta


class InPypElement(PypElement):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def grasp(self):
        """
        Every input pipe need to implement this must have a yield
        in it
        :return:
        """


class OutPypElement(PypElement):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def extract(self, data):
        """
        Every output pipe need to implement this, like outputting
        to file or db

        :param data:
        :return:
        """

    @abc.abstractmethod
    def close(self):
        """
        Close the pyp, like a_file.close() or stop db connection

        :return:
        """


class FilterPypElement(MidPypElement):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def stay(self, data):
        """
        Every filter pipe needs to implement this
        return true to have the data
        return false to decline it

        :param data:
        :return:
        """


class ExtendPypElement(MidPypElement):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def extend(self, data):
        """
        Every filter pipe needs to implement this
        return true or false

        :param data:
        :return:
        """



# -*- coding: utf-8 -*-
"""
pyplyn.pyp
~~~~~~~~~~

This module implements the Pyplyn Pyp aka Pipe

Author: Taha Dogan Gunes
License: MIT, see LICENSE for more details.

"""
from . import elements


class Pipe(object):
    """
    The class handles pipe elements, runs them.
    """

    def __init__(self):
        self.elements = []

    def add(self, element):
        """
        Adds a pipe element to pipe
        :param element:
        :return:
        """
        self.elements.append(element)

    @staticmethod
    def make_assertions(input_pipe, other_pipes, output_pipe):
        """
        To assure that the pipe is correctly settled
        :param input_pipe:
        :param other_pipes: can be []
        :param output_pipe:
        :return:
        """
        assert isinstance(input_pipe, elements.InPypElement), 'Wrong input element type, want a InPypElement!'
        assert isinstance(output_pipe, elements.OutPypElement), 'Wrong output element type, want a OutPypElement!'
        for other_pipe in other_pipes:
            assert isinstance(other_pipe, elements.MidPypElement), 'Wrong middle element type, want a MidPypElement!'


    def run(self):
        """
        Let the pipe flow
        :return:
        """
        assert len(self.elements) >= 2, "In order flow, pipe needs 2 or more elements"
        in_pipe = self.elements[0]
        other_pipes = self.elements[1:-1]
        out_pipe = self.elements[-1]

        self.make_assertions(in_pipe, other_pipes, out_pipe)

        for data in in_pipe.grasp():
            write = True

            for element in other_pipes:
                if isinstance(element, elements.ExtendPypElement):
                    data = element.extend(data)
                elif isinstance(element, elements.FilterPypElement):
                    if not element.stay(data):
                        write = False
                        break
            if write:
                out_pipe.extract(data)



# -*- coding: utf-8 -*-

"""
pyplyn, only one-way pipeline
~~~~~~~~~~~~~~~~~~~~~~~~~~~~

"""
__title__ = 'pyplyn'
__version__ = '0.1.4'
__author__ = 'Taha Dogan Gunes'
__license__ = 'MIT'
__copyright__ = 'Copyright 2014 Taha Dogan Gunes'

# Set default logging handler to avoid "No handler found" warnings.

from .elements import ExtendPypElement, MidPypElement, OutPypElement, InPypElement, FilterPypElement, PypElement
from .inputs import LineReader, ListGiver
from .middles import LambdaExtension, LambdaFilter, Negation, UniqueFilter
from .outputs import LineWriter, Writer, Printer
from .pyp import Pipe


import logging

try:  # Python 2.7+
    from logging import NullHandler
except ImportError:
    class NullHandler(logging.Handler):
        def emit(self, record):
            pass

logging.getLogger(__name__).addHandler(NullHandler())



# -*- coding: utf-8 -*-
"""
pyplyn.outputs
~~~~~~~~~~

Some useful output classes

Author: Taha Dogan Gunes
License: MIT, see LICENSE for more details.

"""

from . import elements

class Writer(elements.OutPypElement):
    """
    Writes the data to a text file
    """

    def __init__(self, path):
        self.output_file = open(path, 'w')

    def extract(self, data):
        self.output_file.write(data)

    def close(self):
        self.output_file.close()


class LineWriter(Writer):
    """
    Writes the data per line
    """

    def extract(self, data):
        super(LineWriter, self).extract(data + "\n")

class Printer(elements.OutPypElement):
    """
    Simple data printer
    """
    def close(self):
        pass

    def extract(self, data):
        print data


class Gatherer(elements.OutPypElement):
    """
    Outputs are gathered in to the pool inside this gatherer class
    after pipe is totally flowed, you can access the data by accesing to
    self.pool list
    """
    def __init__(self):
        self.pool = []

    def extract(self, data):
        self.pool.append(data)

    def close(self):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pyplyn

from distutils.core import setup

setup(
    name="pyplyn",
    version=pyplyn.__version__,
    author='Taha Dogan Gunes',
    include_package_data=True,
    author_email='tdgunes@gmail.com',
    url='https://github.com/tdgunes/pyplyn',
    description='Only one-way pipeline (pyplyn) for data handling in Python',
    packages=['pyplyn',],
    license='MIT',
    long_description=open('README.rst').read(),
)